@@ -1,47 +1,27 @@
 # -*- coding: utf-8 -*-
 from __future__ import print_function
+from functools import wraps
 from importlib.machinery import all_suffixes
 
 import abc
+import collections
+import collections.abc
+import inspect
 import itertools
 import sys, os
 import warnings
 import weakref
 
-from clu.constants.consts import λ, φ, BASEPATH, PROJECT_NAME, NoDefault, pytuple
-from clu.constants.exceptions import ExportError, ExportWarning
-from clu.constants.polyfills import MutableMapping, lru_cache
+chain = itertools.chain
+iterchain = itertools.chain.from_iterable
 
-def doctrim(docstring):
-    """ This function is straight outta PEP257 -- q.v. `trim(…)`,
-       “Handling Docstring Indentation” subsection sub.:
-            https://www.python.org/dev/peps/pep-0257/#id18
-    """
-    from clu.constants.consts import MAXINT
-    
-    if not docstring:
-        return ''
-    # Convert tabs to spaces (following the normal Python rules)
-    # and split into a list of lines:
-    lines = docstring.expandtabs().splitlines()
-    # Determine minimum indentation (first line doesn't count):
-    indent = MAXINT
-    for line in lines[1:]:
-        stripped = line.lstrip()
-        if stripped:
-            indent = min(indent, len(line) - len(stripped))
-    # Remove indentation (first line is special):
-    trimmed = [lines[0].strip()]
-    if indent < MAXINT:
-        for line in lines[1:]:
-            trimmed.append(line[indent:].rstrip())
-    # Strip off trailing and leading blank lines:
-    while trimmed and not trimmed[-1]:
-        trimmed.pop()
-    while trimmed and not trimmed[0]:
-        trimmed.pop(0)
-    # Return a single string:
-    return '\n'.join(trimmed)
+from clu.constants.consts import λ, φ, BASEPATH, PROJECT_NAME, NoDefault, pytuple # type: ignore
+from clu.constants.exceptions import ExportError, ExportWarning
+from clu.constants.polyfills import lru_cache # type: ignore
+from clu.abstract import Slotted, ValueDescriptor, Prefix
+
+# Q.v. `search_by_id(…)` function sub.
+cache = lambda function: lru_cache(maxsize=128, typed=False)(function)
 
 def itermodule(module):
     """ Get an iterable of `(name, thing)` tuples for all things
@@ -76,9 +56,6 @@ def itermoduleids(module):
     ids = (id(getattr(module, key)) for key in keys)
     return zip(keys, ids)
 
-# Q.v. `search_by_id(…)` function sub.
-cache = lru_cache(maxsize=128, typed=False)
-
 # This goes against all logic and reason, but it fucking seems
 # to fix the problem of constants, etc showing up erroneously
 # as members of the `__console__` or `__main__` modules –
@@ -162,6 +139,12 @@ def determine_name(thing, name=None, try_repr=False):
     # … or if called without a “thing”:
     if thing is None:
         return None
+    # Check for a function wrapper:
+    if callable(thing):
+        # N.B. when the Walrus hits Python main this’ll be
+        # a good spot for one of those:
+        if callable(getattr(thing, '__wrapped__', None)):
+            return determine_name(thing.__wrapped__)
     # Check for telltale function-object attributes:
     code = None
     if hasattr(thing, '__code__'): # Python 3.x
@@ -175,10 +158,10 @@ def determine_name(thing, name=None, try_repr=False):
             name = code.co_name
     # … Otherwise, try the standard name attributes:
     if name is None:
-        if hasattr(thing, '__qualname__'):
-            name = thing.__qualname__
-        elif hasattr(thing, '__name__'):
+        if hasattr(thing, '__name__'):
             name = thing.__name__
+        elif hasattr(thing, '__qualname__'):
+            name = thing.__qualname__
     # We likely have something by now:
     if name is not None:
         return name
@@ -191,13 +174,61 @@ def determine_name(thing, name=None, try_repr=False):
     # up returning None:
     return search_for_name(thing)
 
+class rename(object):
+    
+    """ Function-rename decorator. Use like so:
+        
+            @rename(named='yodogg')
+            def YoDogg(*args):
+                ...
+        
+        … or:
+            
+            yodogg = lambda *args: ...
+            yodogg = rename()(yodogg) # awkward syntax, I know
+        
+    """
+    
+    def __init__(self, named=None,
+                        path=None,
+                     dotpath=None):
+        """ Initialize a @rename object decorator. All parameters are optional. """
+        self.named = named
+        self.dotpath = dotpath or path_to_dotpath(path,
+                                                  relative_to=BASEPATH)
+    
+    def assign_name(self, function, name=None):
+        """ Assign the function’s new name. Returns the mutated function. """
+        named = determine_name(function, name=name or self.named)
+        dname = getattr(function, '__name__')
+        if dname in (λ, φ):
+            if named in (λ, φ):
+                named = search_for_name(function)
+            if named is None:
+                raise NameError(str(id(function)))
+            function.__name__ = function.__qualname__ = named
+            function.__lambda_name__ = dname # To recall the lambda’s genesis
+            if dname == φ and self.dotpath is not None:
+                function.__module__ = str(self.dotpath) # Reset __module__ for phi-types
+        return function
+    
+    def __call__(self, f):
+        function = self.assign_name(f)
+        if not inspect.isfunction(function):
+            return function
+        @wraps(function)
+        def renamed(*args, **kwargs):
+            return function(*args, **kwargs)
+        return function
+
 # N.B. Items in the “replaceable_endings” tuple that
 # possibly contain other such items should appear
 # *before* the items that they contain, e.g.:
+ending_suffixes = tuple(all_suffixes())
 ending_prefixes = tuple(f"{os.extsep}{name}" \
                                   for name \
                                    in pytuple('init', 'main'))
-ending_suffixes = tuple(all_suffixes())
+
 replaceable_endings  = tuple(f"{pre}{suf}" \
                             for pre, suf \
                              in itertools.product(ending_prefixes,
@@ -243,74 +274,9 @@ def path_to_dotpath(path, relative_to=None):
     
     return dotpath
 
-def predicates_for_types(*types):
-    """ For a list of types, return a list of “isinstance” predicates """
-    predicates = []
-    for classtype in frozenset(types):
-        predicates.append(lambda thing: isinstance(thing, classtype))
-    return tuple(predicates)
-
-class Slotted(abc.ABCMeta):
-    
-    """ A metaclass that ensures its classes, and all subclasses,
-        will be slotted types.
-    """
-    
-    def __new__(metacls, name, bases, attributes, **kwargs):
-        """ Override for `abc.ABCMeta.__new__(…)` setting up a
-            derived slotted class.
-        """
-        if '__slots__' not in attributes:
-            attributes['__slots__'] = tuple()
-        
-        return super(Slotted, metacls).__new__(metacls, name,
-                                                        bases,
-                                                        attributes,
-                                                      **kwargs)
-
-class ValueDescriptor(object):
-    
-    __slots__ = ('value',)
-    
-    def __init__(self, value):
-        self.value = value
-    
-    def __get__(self, *args):
-        return self.value
-    
-    def __set__(self, instance, value):
-        pass
-    
-    def __repr__(self):
-        from clu.constants.consts import ENCODING
-        return isinstance(self.value, str)   and self.value or \
-               isinstance(self.value, bytes) and self.value.decode(ENCODING) or \
-                                            repr(self.value)
-
-class Prefix(Slotted):
-    
-    """ A metaclass to assign a “prefix” class property,
-        extracted from a “prefix” class keyword, to a new
-        slotted type. 
-    """
-    
-    @classmethod
-    def __prepare__(metacls, name, bases, prefix="/", **kwargs):
-        """ Remove the “prefix” class keyword before calling up """
-        return super(Prefix, metacls).__prepare__(name, bases, **kwargs)
-    
-    def __new__(metacls, name, bases, attributes, prefix="/", **kwargs):
-        """ Override for `Slotted.__new__(…)` setting up a
-            derived slotted class that pulls from a “prefix”
-            with the requisite methods defined for access.
-        """
-        if 'prefix' not in attributes:
-            attributes['prefix'] = ValueDescriptor(prefix)
-        
-        return super(Prefix, metacls).__new__(metacls, name,
-                                                       bases,
-                                                       attributes,
-                                                     **kwargs)
+# One-off private versions of “ismergeable” and “isstring” – NOT FOR EXPORT:
+ismergeable = lambda thing: isinstance(thing, collections.abc.Mapping)
+isstring = lambda thing: isinstance(thing, str)
 
 classes = {}
 appnames = set()
@@ -319,7 +285,7 @@ class Registry(abc.ABC, metaclass=Slotted):
     
     """ A class-registry mixin ancestor type suitable for use
         in the ExporterBase inheritance chain – it uses the
-        “clu.exporting.Slotted” metaclass and respects the
+        “clu.abstract.Slotted” metaclass and respects the
         class keywords already in use.
     """
     
@@ -332,25 +298,29 @@ class Registry(abc.ABC, metaclass=Slotted):
         if appname in classes:
             raise TypeError(f"appname already registered: {appname}")
         classes[appname] = cls
-        super(Registry, cls).__init_subclass__(**kwargs)
+        super(Registry, cls).__init_subclass__(**kwargs) # type: ignore
         cls.instances = weakref.WeakValueDictionary()
         cls.appname = ValueDescriptor(appname)
     
     @staticmethod
     def all_appnames():
         """ Return a tuple of all registered appnames """
-        return tuple(sorted(appnames))
+        return sorted(appnames)
     
     @staticmethod
     def for_appname(appname):
         """ Return a subclass for a registered appname """
-        from clu.typology import isstring
         if not appname:
             raise ValueError("appname required")
         if not isstring(appname):
             raise TypeError("class registry access by string keys only")
         return classes[appname]
     
+    @staticmethod
+    def has_appname(appname):
+        """ Check to see if a given appname has a registered subclass """
+        return (appname in appnames) and (appname in classes)
+    
     @classmethod
     def __class_getitem__(cls, key):
         """ Return a specific registered class from the registry,
@@ -361,17 +331,27 @@ class Registry(abc.ABC, metaclass=Slotted):
         """
         return cls.for_appname(key)
     
-    @classmethod
-    def module_getters(cls):
-        from clu.predicates import attr_across
-        return attr_across('modules', *[classes[appname] \
-                           for appname in cls.all_appnames()])
+    @staticmethod
+    def unregister(appname):
+        """ Unregister a previously-registered appname, returning the
+            successfully unregistered ExporterBase subclass.
+            
+            Attempting to unregister the core CLU application’s exporter
+            is not allowed and will raise a KeyError.
+        """
+        if appname == PROJECT_NAME:
+            raise KeyError("Can’t unregister the core application exporter")
+        cls = classes.pop(appname, None)
+        if cls:
+            if appname in appnames:
+                appnames.remove(appname)
+        return cls
     
     @classmethod
     def all_modules(cls):
-        from itertools import chain
-        return tuple(chain.from_iterable(modules().values() \
-                     for modules in cls.module_getters()))
+        return iterchain(modules().values() \
+                     for modules in (classes[appname].modules \
+                     for appname in cls.all_appnames()))
     
     @classmethod
     def nameof(cls, thing):
@@ -387,13 +367,13 @@ class Registry(abc.ABC, metaclass=Slotted):
         """
         return search_modules(thing, *cls.all_modules())[0]
 
-class ExporterBase(MutableMapping, Registry, metaclass=Prefix):
+class ExporterBase(collections.abc.MutableMapping, Registry, metaclass=Prefix):
     
     """ The base class for “clu.exporting.Exporter”. Override this
         class in your own project to use the CLU exporting mechanism –
         q.v. “clu.exporting.Exporter” docstring sub.
         
-        This class uses the “clu.exporting.Prefix” metaclass, which
+        This class uses the “clu.abstract.Prefix” metaclass, which
         automatically adds a “prefix” class attribute, as well as
         the “clu.exporting.Registry” mixin, which keeps a registry
         containing it and all of its derived subclasses, and furnishes
@@ -402,15 +382,16 @@ class ExporterBase(MutableMapping, Registry, metaclass=Prefix):
     
     __slots__ = pytuple('exports', 'weakref') + ('path', 'dotpath')
     
-    def __new__(cls, *args, path=None, **kwargs):
+    def __new__(cls, *args, path=None, dotpath=None, **kwargs):
         try:
-            instance = super(ExporterBase, cls).__new__(cls, *args, **kwargs)
+            instance = super(ExporterBase, cls).__new__(cls, *args, **kwargs) # type: ignore
         except TypeError:
             instance = super(ExporterBase, cls).__new__(cls)
         
         instance.__exports__ = {}
         instance.path = path
-        instance.dotpath = path_to_dotpath(path, relative_to=cls.prefix)
+        instance.dotpath = path_to_dotpath(path,
+             relative_to=cls.prefix) or dotpath
         
         if instance.dotpath is not None:
             cls.instances[instance.dotpath] = instance
@@ -432,19 +413,28 @@ class ExporterBase(MutableMapping, Registry, metaclass=Prefix):
         # already spent more than enough time fucking around with
         # “typing” shit and I am not interested at this time, and
         # b) it’s too useful a method to give it all up. So deal.
-        from clu.typology import isstring
         if not key:
             raise ValueError("instance key required")
         if not isstring(key):
             raise TypeError("instance registry access by string keys only")
         return cls.instances[key]
     
+    @classmethod
+    def unregister(cls, dotpath):
+        """ Unregister a previously-registered ExporterBase instance,
+            specified by the dotted path (née “dotpath”) of the module
+            in which it is ensconced.
+            
+            Returns the successfully unregistered instance in question.
+        """
+        return cls.instances.pop(dotpath, None)
+    
     @classmethod
     def modulenames(cls):
         """ Get a sorted list of module names – the keys to the Exporter
             instance registry – that are currently available
         """
-        return tuple(sorted(cls.instances.keys()))
+        return sorted(cls.instances.keys())
     
     @classmethod
     def modules(cls):
@@ -474,7 +464,7 @@ class ExporterBase(MutableMapping, Registry, metaclass=Prefix):
         """
         return search_modules(thing, *cls.modules().values())[0]
     
-    def __init__(self, *args, path=None, **kwargs):
+    def __init__(self, *args, path=None, dotpath=None, **kwargs):
         for arg in args:
             if hasattr(arg, '__exports__'):
                 self.__exports__.update(arg.__exports__)
@@ -492,7 +482,7 @@ class ExporterBase(MutableMapping, Registry, metaclass=Prefix):
     
     def exports(self):
         """ Get a new dictionary instance filled with the exports. """
-        out = {}
+        out = {} # type: dict
         out.update(self.__exports__)
         return out
     
@@ -595,7 +585,7 @@ class ExporterBase(MutableMapping, Registry, metaclass=Prefix):
         # a settable __doc__ slot or established attribute).
         if doc is not None:
             try:
-                thing.__doc__ = doctrim(doc)
+                thing.__doc__ = inspect.cleandoc(doc)
             except (AttributeError, TypeError):
                 typename = determine_name(type(thing))
                 warnings.warn(type(self).messages['docstr'] % (named, typename),
@@ -626,11 +616,11 @@ class ExporterBase(MutableMapping, Registry, metaclass=Prefix):
     
     def all_tuple(self, *additionals):
         """ For use in module `__all__` tuple definitions """
-        return tuple(self.keys()) + tuple(additionals)
+        return tuple(chain(self.keys(), additionals))
     
     def dir_function(self, *additionals):
         """ Return a list containing the exported module names. """
-        return lambda: list(tuple(self.keys()) + tuple(additionals))
+        return lambda: list(chain(self.keys(), additionals))
     
     def all_and_dir(self, *additionals):
         """ Assign a modules’ __all__ and __dir__ values, e.g.:
@@ -666,7 +656,7 @@ class ExporterBase(MutableMapping, Registry, metaclass=Prefix):
         return search_by_id.cache_info()
     
     def __iter__(self):
-        return iter(self.__exports__.keys())
+        yield from iter(self.__exports__.keys())
     
     def __len__(self):
         return len(self.__exports__)
@@ -685,7 +675,6 @@ class ExporterBase(MutableMapping, Registry, metaclass=Prefix):
     
     def __add__(self, operand):
         # On add, old values are not overwritten
-        from clu.predicates import ismergeable
         from clu.dicts import merge_two
         if not ismergeable(operand):
             return NotImplemented
@@ -693,7 +682,6 @@ class ExporterBase(MutableMapping, Registry, metaclass=Prefix):
     
     def __radd__(self, operand):
         # On reverse-add, old values are overwritten
-        from clu.predicates import ismergeable
         from clu.dicts import merge_two
         if not ismergeable(operand):
             return NotImplemented
@@ -701,7 +689,6 @@ class ExporterBase(MutableMapping, Registry, metaclass=Prefix):
     
     def __iadd__(self, operand):
         # On in-place add, old values are updated and replaced
-        from clu.predicates import ismergeable
         if not ismergeable(operand):
             return NotImplemented
         self.update(**getattr(operand, '__exports__', operand))
@@ -721,9 +708,9 @@ class ExporterBase(MutableMapping, Registry, metaclass=Prefix):
     
     def __dir__(self):
         return list(filter(lambda name: name not in ('all_appnames',
+                                                     'all_modules',
                                                      'for_appname',
-                                                     'module_getters',
-                                                     'all_modules'),
+                                                     'unregister'),
                            super(ExporterBase, self).__dir__()))
 
 class Exporter(ExporterBase, prefix=BASEPATH, appname=PROJECT_NAME):
@@ -747,7 +734,7 @@ class Exporter(ExporterBase, prefix=BASEPATH, appname=PROJECT_NAME):
         
         Also note that all derived subclasses of ExporterBase will
         automatically be slotted classes – a “__slots__” attribute
-        will be added to the class dict by the “clu.exporting.Slotted”
+        will be added to the class dict by the “clu.abstract.Slotted”
         metaclass, if your subclass doesn’t define one – and so if
         you desire a class with a working “__dict__” attribute for
         some reason, you’ll need to specify:
@@ -762,7 +749,6 @@ class Exporter(ExporterBase, prefix=BASEPATH, appname=PROJECT_NAME):
 exporter = Exporter(path=__file__)
 export = exporter.decorator()
 
-export(doctrim)
 export(itermodule)
 export(moduleids)
 export(itermoduleids)
@@ -771,13 +757,11 @@ export(search_for_name)
 export(search_for_module)
 export(search_modules)
 export(determine_name)
+export(rename)
 export(path_to_dotpath)
-export(predicates_for_types)
 export(sysmods,         name='sysmods',         doc="sysmods() → shortcut for reversed(tuple(frozenset(sys.modules.values()))) …OK? I know. It’s not my finest work, but it works.")
 
 # NO DOCS ALLOWED:
-export(Slotted)
-export(ValueDescriptor)
 export(Registry)
 export(ExporterBase)
 export(Exporter)        # hahaaaaa
