@@ -1,123 +1,52 @@
 # -*- coding: utf-8 -*-
 from __future__ import print_function
-from itertools import chain
-
-iterchain = chain.from_iterable
+from functools import lru_cache
 
 import os
 import re
 import sys
 
-from clu.constants.consts import ENCODING, SINGLETON_TYPES
-from clu.constants.polyfills import lru_cache
-from clu.predicates import negate, ismetaclass, typeof, or_none, isenum, enumchoices, pyname
-from clu.typology import isvalidpath, isnumeric, isbytes, isstring
+from clu.constants.consts import ENCODING
+from clu.predicates import negate, true_function
+from clu.typology import isvalidpath, isnumeric, isstring
 from clu.exporting import Exporter
 
 exporter = Exporter(path=__file__)
 export = exporter.decorator()
 
+cache = lambda function: export(lru_cache(maxsize=32)(function))
+onecache = lambda function: export(lru_cache(maxsize=1)(function))
+
 # OS UTILITIES: get the current users’ home directory
 
 gethomedir = lambda: os.path.expanduser("~")
 isinvalidpath = negate(isvalidpath)
 
-@export
-def wrap_value(value):
-    """ Get a “lazified” copy of a value, wrapped in a lamba """
-    wrapper = lambda *args, **kwargs: value
-    wrapper.__wrapped__ = value
-    return wrapper
-
-none_function = wrap_value(None)
-true_function = wrap_value(True)
-
-hexid = lambda thing: hex(id(thing))
-typenameof = lambda thing: pyname(typeof(thing))
-typename_hexid = lambda thing: (pyname(typeof(thing)), hex(id(thing)))
-
-def stringify_value(v):
-    """ Basic, simple, straightforward type-switch-based sub-repr """
-    T = type(v)
-    if isstring(T):
-        return f"“{v}”"
-    elif T in SINGLETON_TYPES:
-        return str(v)
-    elif isnumeric(T):
-        return str(v)
-    elif isbytes(T):
-        return f"“{v.decode(ENCODING)}”"
-    elif ismetaclass(T):
-        if isenum(v):
-            typename, hex_id = typename_hexid(v)
-            choices = ", ".join(enumchoices(v))
-            return f"‘{typename}<{v.__name__}({choices}) @ {hex_id}>’"
-        return repr(v)
-    return f"‘{v!r}’"
-
-@export
-def stringify(instance, fields,
-                       *extras, try_callables=True,
-                      **attributes):
-    """ Stringify an object instance, using an iterable field list to
-        extract and render its values, and printing them along with the 
-        typename of the instance and its memory address -- yielding a
-        repr-style string of the format:
-        
-            TypeName(fieldname="val", otherfieldname="otherval") @ 0x0FE
-        
-        The `stringify(…)` function is of use in `__str__()` and `__repr__()`
-        definitions, e.g. something like:
-        
-            def __repr__(self):
-                return stringify(self, type(self).__slots__)
-        
-        Callable fields, by default, will be called with no arguments
-        to obtain their value. To supress this behavior – if you wish
-        to represent callable fields that require arguments – you can
-        pass the keyword-only “try_callables” flag as False:
-            
-            def __repr__(self):
-                return stringify(self,
-                            type(self).__slots__,
-                            try_callables=False)
-    """
-    typename, hex_id = typename_hexid(instance)
-    if all(len(param) < 1 for param in (fields, extras, attributes)):
-        return f"{typename}(¬) @ {hex_id}"
-    attrs = dict(attributes)
-    for field in chain(fields, extras):
-        field_value = or_none(instance, field)
-        if try_callables and callable(field_value):
-            field_value = field_value()
-        if field_value:
-            attrs[field] = stringify_value(field_value)
-    attr_string = ", ".join(f'{k}={v}' for k, v in attrs.items()) or "…"
-    return f"{typename}({attr_string}) @ {hex_id}"
-
 ex = os.path.extsep
 dolla = '$'
 
-@export
-@lru_cache(maxsize=32)
+@cache
 def re_matcher(string):
     """ Return a boolean function that will search for the given
         regular-expression within any strings with which it is called,
         returning True when the regex matches from the beginning of the
         string, and False when it doesn’t.
+        
+        Results are cached, via “functools.lru_cache(…)”
     """
     if not string:
         return true_function
     match_function = re.compile(string, re.IGNORECASE).match
     return lambda searching: bool(match_function(searching))
 
-@export
-@lru_cache(maxsize=32)
+@cache
 def re_searcher(string):
     """ Return a boolean function that will search for the given
         regular-expression within any strings with which it is called,
         returning True when the regex matches and False when it doesn’t.
         
+        Results are cached, via “functools.lru_cache(…)”
+        
         Useful in filter(…) calls and comprehensions, e.g.:
         
         >>> plists = filter(re_searcher(r'.plist$'), os.listdir())
@@ -134,7 +63,7 @@ def re_suffix(string):
     """ Remove any “os.path.extsep” prefixing a string and ensure
         it ends with a “$” – to indicate a regular expression suffix.
     """
-    if string is None:
+    if not string:
         return None
     return rf"{string.lower().lstrip(ex).rstrip(dolla)}$"
 
@@ -268,10 +197,9 @@ def win32_longpath(path):
 
 octalize = lambda integer: "0o%04o" % integer
 
-@export
-@lru_cache(maxsize=1)
+@onecache
 def current_umask():
-    """ Get the current umask value (cached on Python 3 and up). """
+    """ Get the current umask value. Results are cached, via “functools.lru_cache(…)” """
     mask = os.umask(0)
     os.umask(mask)
     return mask
@@ -295,14 +223,7 @@ def masked_chmod(path, perms=0o666):
 # MODULE EXPORTS:
 export(gethomedir,              name='gethomedir',          doc="gethomedir() → Return the current user’s home directory")
 export(isinvalidpath,           name='isinvalidpath',       doc="isinvalidpath(thing) → boolean predicate, True if `thing` does not represent a valid path on the filesystem")
-export(none_function,           name='none_function',       doc="none_function() → A function that always returns None")
-export(true_function,           name='true_function',       doc="true_function() → A function that always returns True")
-
-export(hexid,                   name='hexid',               doc="hexid(thing) → Return the hex-ified representation of “thing”’s ID – Equivalent to “hex(id(thing))”")
-export(typenameof,              name='typenameof',          doc="typenameof(thing) → Return the string name of the type of “thing” – Equivalent to “pyname(typeof(thing))”, q.v. “clu.predicates”")
-export(typename_hexid,          name='typename_hexid',      doc="typename_hexid(thing) → Return a two-tuple containing “thing”’s hex-ified ID and the string name of the type of “thing” – Equivalent to “(hexid(thing), typenameof(thing))”")
 export(differentfile,           name='differentfile',       doc="differentfile(path0, path1) → Return True if path0 and path1 point to different locations on the filesystem")
-
 export(octalize,                name='octalize',            doc="octalize(integer) → Format an integer value as an octal number")
 
 # Assign the modules’ `__all__` and `__dir__` using the exporter:
