@@ -10,10 +10,6 @@ from clu.exporting import Exporter, Registry
 exporter = Exporter(path=__file__)
 export = exporter.decorator()
 
-# MODULE SEARCH FUNCTIONS: iterate and search modules, yielding
-# names, thing values, and/or id(thing) values, matching by given
-# by thing names or id(thing) values
-
 @export
 def determine_module(thing, name=None):
     """ Private module function to find the module of a thing,
@@ -26,14 +22,56 @@ def determine_module(thing, name=None):
     if thing is None:
         return None
     
-    return pickle.whichmodule(thing, None)
+    return pickle.whichmodule(thing, None) or name # type: ignore
+
+"""
+NAME AND MODULE SEARCH FUNCTIONS: the “nameof(…)” and “moduleof(…)”
+functions each search through, in order:
+  
+  1) the object itself for relevant attributes (e.g “__name__”
+     or “__module__”) – this is nearly instantaneous;
+  
+  2) The “clu.exporting” registry of Exporter classes and their
+     registered instances – this is very fast, equivalent to an
+     iteration over the values in a sequence of dicts;
+  
+  3) The entirety of all currently loaded module namespaces,
+     as per “sys.modules” – this is potentially a large search
+     space (the virtualenv I am using now to develop this code
+     currently has 1235 unique modules loaded up); we use the
+     standard-library “@functools.lru_cache” decorator to
+     cache the ID values of the objects for which we search
+     to return near-instantaneous results for repeat queries
+     without much overhead.
+
+… The “nameof(…)” and “moduleof(…)” functions in “clu.naming”
+are intended as a top-level, users-of-CLU-facing API. They each
+call functions that are also available to users (like e.g. the
+“pyname(…)” and “pymodule(…)” functions from “clu.predicates”
+and the eponymous class methods of “clu.exporting.Registry”).
+
+One caveat is that a number of lower-level functions returning
+a module for an object will return the actual module instance
+itself, rather than a string name or qualified name value.
+
+The functions “determine_name(…)” and “determine_module(…)”,
+while exported publicly, are intended as private utilities for
+which no guarantees are made about how they work, their arity,
+their time complexity, or their very existence.
+
+In general, these functions will work as fast as possible –
+near attribute-access speed – for 99.9 percent of use-cases;
+the few remainig 0.99 percent will be nearly as fast, and for
+that pathological 0.001-percent case… well, that may be an
+indication that you need to change something, dogg.
+"""
 
 @export
 def nameof(thing, default=NoDefault):
-    """ Get the name of a thing, according to either:
-        >>> thing.__qualname__
-        … or:
-        >>> thing.__name__
+    """ Get the name of a thing, according to its attributes,
+        how it appears as a registered item in any Exporter
+        subclasses, or (failing any of those) as it appears
+        in the module in which it appears to be ensconced –
         … optionally specifying a “default” fallback.
     """
     from clu.predicates import pyname
@@ -49,7 +87,8 @@ def nameof(thing, default=NoDefault):
 @export
 def moduleof(thing, default=NoDefault):
     """ Determine in which module a given thing is ensconced,
-        and return that modules’ name as a string.
+        and return that modules’ name as a string
+        … optionally specifying a “default” fallback.
     """
     from clu.predicates import pymodule
     
@@ -63,6 +102,50 @@ def moduleof(thing, default=NoDefault):
         return result
     return result or default
 
+# MODULE INSPECTOR PREDICATES: determine what undergirds a module object
+
+isbuiltin = lambda thing: moduleof(thing) == 'builtins'
+
+@export
+def isnativemodule(module):
+    """ isnativemodule(thing) → boolean predicate, True if `module`
+        is a native-compiled (“extension”) module.
+        
+        Q.v. this fine StackOverflow answer on this subject:
+            https://stackoverflow.com/a/39304199/298171
+    """
+    import importlib.machinery, inspect, os
+    from clu.predicates import getpyattr
+    from clu.typology import ismodule
+    
+    # Step one: modules only beyond this point:
+    if not ismodule(module):
+        return False
+    
+    # Step two: return truly when “__loader__” is set:
+    if isinstance(getpyattr(module, 'loader'),
+                  importlib.machinery.ExtensionFileLoader):
+        return True
+    
+    # Step three: in leu of either of those indicators,
+    # check the module path:
+    pth = inspect.getfile(module)
+    ext = os.path.splitext(pth)[1]
+    return ext in importlib.machinery.EXTENSION_SUFFIXES
+
+@export
+def isnative(thing):
+    """ isnative(thing) → boolean predicate, True if `thing`
+        comes from a native-compiled (“extension”) module.
+    """
+    import importlib
+    module = moduleof(thing)
+    if module == 'builtins':
+        return False
+    return isnativemodule(
+           importlib.import_module(
+                            module))
+
 # QUALIFIED-NAME FUNCTIONS: import by qualified name (like e.g. “yo.dogg.DoggListener”),
 # assess a thing’s qualified name, etc etc.
 
@@ -177,7 +260,7 @@ def split_abbreviations(s):
         If you still find this function inscrutable,
         have a look here: https://gist.github.com/4027079
     """
-    abbreviations = []
+    abbreviations = [] # type: list
     current_token = ''
     for char in s.split(';')[0]:
         if current_token == '':
@@ -195,5 +278,7 @@ def split_abbreviations(s):
             abbreviations.append(current_token)
     return tuple(abbreviations)
 
+export(isbuiltin,      name='isbuiltin',  doc="isbuiltin(thing) → boolean predicate, True if `thing` is a builtin function/method/class")
+
 # Assign the modules’ `__all__` and `__dir__` using the exporter:
 __all__, __dir__ = exporter.all_and_dir()
