@@ -53,8 +53,7 @@ from clu.constants.terminalsize import get_terminal_size
 from clu.compilation import (Macro, Macros,
                              CDBSubBase, CDBBase, CDBJsonFile)
 
-from clu.exporting import (doctrim,
-                           itermodule, moduleids, itermoduleids,
+from clu.exporting import (itermodule, moduleids, itermoduleids,
                            search_by_id,
                            search_for_name, search_for_module,
                            search_modules,
@@ -70,15 +69,18 @@ from clu.extending import (Extensible,
 
 from clu.importing import (Module, Finder, Loader,
                            initialize_types,
+                           all_registered_appnames,
+                           all_registered_modules,
+                           modules_for_appname,
                            ModuleBase, FinderBase, LoaderBase,
                            Package, ModuleSpec,
                            Registry as ClassModuleRegistry,
-                           ArgumentSink)
+                           ArgumentSink, SubModule)
 
 from clu.sanitizer import sanitize, sanitizers, utf8_encode, utf8_decode
 from clu.version import version_info
 
-from clu.predicates import (negate,
+from clu.predicates import (negate, reverse,
                             ismetaclass, isclass, isclasstype,
                             metaclass, typeof,
                             noattr, haspyattr, nopyattr,
@@ -96,10 +98,12 @@ from clu.predicates import (negate,
                             attr_across, stattr_across, pyattr_across, item_across,
                             finditem, finditems,
                             try_items,
-                            dunder_or, mro, unwrap, origin,
-                            isancestor, isorigin,
+                            dunder_or, mro, rmro, unwrap, origin,
+                            isancestor, isorigin, newtype,
                             isenum, enumchoices,
-                            predicate_nop, function_nop, uncallable,
+                            predicate_nop, function_nop,
+                            wrap_value, none_function, true_function,
+                            uncallable, hoist,
                             pyname, pymodule,
                             isexpandable, isnormative, iscontainer,
                             lambda_repr,
@@ -110,7 +114,8 @@ from clu.predicates import (negate,
                             isslotted, isdictish, isslotdicty,
                             case_sort,
                             itervariadic,
-                            tuplize, uniquify, listify,
+                            tuplize, uniquify, listify, union,
+                            ancestral, ancestral_union,
                             allof, anyof, noneof,
                             slots_for)
 
@@ -152,7 +157,7 @@ from clu.config.abc import NAMESPACE_SEP, FlatOrderedSet, NamespacedMutableMappi
 from clu.config.base import Flat, Nested
 from clu.config.env import Env
 from clu.config.filebase import FileName, FileBase
-from clu.config.fieldtypes import ValidationError, hoist
+from clu.config.fieldtypes import ValidationError
 from clu.config.fieldtypes import functional_and, functional_set
 from clu.config.fieldtypes import FieldBase
 from clu.config.fieldtypes import fields
@@ -200,6 +205,7 @@ from clu.enums import (DUNDER, SUNDER,
 
 from clu.repl.ansi import (print_separator, evict_announcer,
                            print_ansi, print_ansi_centered,
+                           paragraphize,
                            ansidoc, highlight,
                            ANSIBase, ANSI,
                            Text, Weight, Background, ANSIFormat)
@@ -305,7 +311,7 @@ __all__ = ('Image',
            'sanitize', 'sanitizers', 'utf8_encode', 'utf8_decode',
            'Macro', 'Macros',
            'CDBSubBase', 'CDBBase', 'CDBJsonFile',
-           'doctrim', 'itermodule', 'moduleids', 'itermoduleids',
+           'itermodule', 'moduleids', 'itermoduleids',
            'search_by_id',
            'search_for_name', 'search_for_module',
            'search_modules',
@@ -319,11 +325,14 @@ __all__ = ('Image',
            'DoubleDutchFunction',
            'Module', 'Finder', 'Loader',
            'initialize_types',
+           'all_registered_appnames',
+           'all_registered_modules',
+           'modules_for_appname',
            'ModuleBase', 'FinderBase', 'LoaderBase',
            'Package', 'ModuleSpec',
            'ClassModuleRegistry',
-           'ArgumentSink',
-           'negate',
+           'ArgumentSink', 'SubModule',
+           'negate', 'reverse',
            'ismetaclass', 'isclass', 'isclasstype',
            'metaclass', 'typeof',
            'noattr', 'haspyattr', 'nopyattr',
@@ -341,10 +350,12 @@ __all__ = ('Image',
            'attr_across', 'stattr_across', 'pyattr_across', 'item_across',
            'finditem', 'finditems',
            'try_items',
-           'dunder_or', 'mro', 'unwrap', 'origin',
-           'isancestor', 'isorigin',
+           'dunder_or', 'mro', 'rmro', 'unwrap', 'origin',
+           'isancestor', 'isorigin', 'newtype',
            'isenum', 'enumchoices',
-           'predicate_nop', 'function_nop', 'uncallable',
+           'predicate_nop', 'function_nop',
+           'wrap_value', 'none_function', 'true_function',
+           'uncallable', 'hoist',
            'pyname', 'pymodule',
            'isexpandable', 'isnormative', 'iscontainer',
            'lambda_repr',
@@ -356,7 +367,8 @@ __all__ = ('Image',
            'slots_for',
            'case_sort',
            'itervariadic',
-           'tuplize', 'uniquify', 'listify',
+           'tuplize', 'uniquify', 'listify', 'union',
+           'ancestral', 'ancestral_union',
            'allof', 'anyof', 'noneof',
            'SimpleNamespace', 'Namespace', 'types', 'modulize',
            'samelength', 'differentlength', 'isunique',
@@ -399,6 +411,7 @@ __all__ = ('Image',
            'DUNDER', 'SUNDER', 'alias', 'AliasingEnumMeta', 'AliasingEnum',
            'print_separator', 'evict_announcer',
            'print_ansi', 'print_ansi_centered',
+           'paragraphize',
            'ansidoc', 'highlight',
            'ANSIBase', 'ANSI',
            'Text', 'Weight', 'Background', 'ANSIFormat',
@@ -462,14 +475,13 @@ try:
                                    TemporaryDirectory, Intermediate,
                                    NamedTemporaryFile)
     from clu.fs.misc import (gethomedir, isinvalidpath,
-                             stringify, stringify_value,
-                             wrap_value,
-                             hexid, typenameof, typename_hexid,
                              re_matcher, re_searcher,
                              re_suffix, suffix_searcher,
                              swapext,
                              differentfile,
                              filesize, samesize, differentsize)
+    from clu.repr import (hexid, typenameof, typename_hexid,
+                          strfield, strfields, stringify)
 except (ImportError, SyntaxError):
     pass
 else:
@@ -486,8 +498,7 @@ else:
                 'TemporaryDirectory', 'Intermediate',
                 'NamedTemporaryFile',
                 'gethomedir', 'isinvalidpath',
-                'stringify', 'stringify_value',
-                'wrap_value',
+                'strfield', 'strfields', 'stringify',
                 'hexid', 'typenameof', 'typename_hexid',
                 're_matcher', 're_searcher',
                 're_suffix', 'suffix_searcher',
